package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.tournament.TournamentDao;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import cz.muni.fi.pa165.services.tournament.TournamentService;
import cz.muni.fi.pa165.services.tournament.TournamentServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek, Marek Kadlencik
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class TournamentServiceTest {

    @Mock
    private TournamentDao tournamentDao;

    @Autowired
    @InjectMocks
    private TournamentService tournamentService = new TournamentServiceImpl();

    private TournamentEntity tournamentA;

    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestTournament() {
        tournamentA = new TournamentEntity("Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        tournamentA.setId(UUID.randomUUID());
    }


    @Test
    public void createTournament() {
        tournamentService.create(tournamentA);
        Mockito.verify(tournamentDao, Mockito.times(1)).create(tournamentA);
    }

    @Test
    public void createNullTournament() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentService.create(null));
    }

    @Test
    public void findTournamentById() {
        when(tournamentDao.findById(tournamentA.getId())).thenReturn(tournamentA);

        TournamentEntity result = tournamentService.findById(tournamentA.getId());

        Assert.assertEquals(result.getId(), tournamentA.getId());
    }

    @Test
    public void findTournamentByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentService.findById(null));
    }

    @Test
    public void findTournamentsByCity() {
        when(tournamentDao.getTournamentsByCity(tournamentA.getCity())).thenReturn(Collections.singletonList(tournamentA));
        List<TournamentEntity> result = tournamentService.getTournamentsByCity(tournamentA.getCity());

        Assert.assertEquals(result.size(), 1);
        Assert.assertTrue(result.contains(tournamentA));
    }

    @Test
    public void findTournamentsByNullCity() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentService.getTournamentsByCity(null));
    }

    @Test
    public void findTournamentsWonBy() {
        TeamEntity teamA = new TeamEntity("Autisti", new HashSet<>(List.of(new TeamCoachEntity("Autista Coach"))));
        when(tournamentDao.getTournamentsWonBy(teamA)).thenReturn(Collections.singletonList(tournamentA));
        List<TournamentEntity> result = tournamentService.getTournamentsWonBy(teamA);

        Assert.assertEquals(result.size(), 1);
        Assert.assertTrue(result.contains(tournamentA));
    }

    @Test
    public void findTournamentsWonByNullTeam() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentService.getTournamentsWonBy(null));
    }

    @Test
    public void findAllTournaments() {
        TournamentEntity tournamentB = new TournamentEntity("Brno", CountryEnum.CZECH_REPUBLIC, 1000, TournamentFormat.DEFAULT, 5);
        when(tournamentDao.findAll()).thenReturn(asList(tournamentB, tournamentA));

        List<TournamentEntity> result = tournamentService.findAll();

        Assert.assertEquals(result.size(), 2);
    }


    @Test
    public void deleteTournament() {
        tournamentService.remove(tournamentA);

        verify(tournamentDao, times(1)).remove(tournamentA);
    }

    @Test
    public void deleteNullTournament() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentService.remove(null));
    }

    @Test
    public void updateTournament() {
        TournamentEntity tournamentB = new TournamentEntity(
                "Brno II", CountryEnum.CZECH_REPUBLIC, 1000, TournamentFormat.DEFAULT, 5);

        tournamentService.update(tournamentB);

        verify(tournamentDao, times(1)).update(tournamentB);
    }

    @Test
    public void updateNullTournament() {
        Assert.assertThrows(IllegalArgumentException.class, () -> tournamentService.update(null));
    }

}
